Package: polycor
Version: 0.8-2
Title: Polychoric and Polyserial Correlations
Authors@R: c(person("John", "Fox", role = "aut"),
             person("Adrian", "Dusa", role = "ctb"),
             person("Duncan", "Murdoch", email = "murdoch.duncan@gmail.com", role = "cre"))
Depends: R (>= 3.3.0)
Imports: stats, mvtnorm, Matrix, parallel, admisc (>= 0.22)
ByteCompile: yes
LazyLoad: yes
Description: Computes polychoric and polyserial correlations by quick "two-step" methods or ML, 
  optionally with standard errors; tetrachoric and biserial correlations are special cases.
License: GPL (>= 2)
URL: https://github.com/dmurdoch/polycor,
        https://CRAN.R-project.org/package=polycor
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-12-13 15:16:03 UTC; murdoch
Author: John Fox [aut],
  Adrian Dusa [ctb],
  Duncan Murdoch [cre]
Maintainer: Duncan Murdoch <murdoch.duncan@gmail.com>
Repository: CRAN
Date/Publication: 2025-12-14 06:10:30 UTC
Built: R 4.6.0; ; 2026-04-26 00:03:52 UTC; unix
