# last modified 2022-01-10 by J. Fox

importFrom("stats", "complete.cases", "cor", "dnorm", "na.omit",
         "optim", "optimise", "pchisq", "pnorm", "qnorm", "sd", "coef")

importFrom("parallel", "detectCores")

importFrom("admisc", "tryCatchWEM")

export(hetcor, polychor, polyserial)

export(detectCores)

S3method(hetcor, data.frame)
S3method(hetcor, default)
S3method(print, polycor)
S3method(print, hetcor)
S3method(as.matrix, hetcor)

S3method(coef, polycor)
S3method(vcov, polycor)
S3method(summary, polycor)
